from datetime import datetime, timedelta

import pymongo
from discord.ext.tasks import loop

from jarvis import jarvis, jarvis_db
from jarvis.db.types import Lock


@loop(minutes=1)
async def unlock():
    locks = Lock.get_active()
    updates = []
    for lock in locks:
        if (
            lock.created_at + timedelta(minutes=lock.duration)
            < datetime.utcnow()
        ):
            guild = await jarvis.fetch_guild(lock.guild)
            channel = await jarvis.fetch_channel(lock.channel)
            if channel:
                roles = await guild.fetch_roles()
                for role in roles:
                    overrides = channel.overwrites_for(role)
                    overrides.send_messages = None
                    await channel.set_permissions(
                        role, overwrite=overrides, reason="Lock expired"
                    )
            updates.append(
                pymongo.UpdateOne(
                    {
                        "channel": channel.id,
                        "guild": guild.id,
                        "created_at": lock.created_at,
                    },
                    {"$set": {"active": False}},
                )
            )
    if updates:
        jarvis_db.locks.bulk_write(updates)
